/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

package org.apache.hyracks.storage.am.btree.impls;

import org.apache.hyracks.api.dataflow.value.ITypeTraits;
import org.apache.hyracks.storage.am.btree.api.IPrefixSlotManager;
import org.apache.hyracks.storage.am.btree.frames.BTreeFieldPrefixNSMLeafFrame;
import org.apache.hyracks.storage.am.common.api.ITreeIndexFrame;
import org.apache.hyracks.storage.am.common.tuples.TypeAwareTupleReference;

public class FieldPrefixPrefixTupleReference extends TypeAwareTupleReference {

    public FieldPrefixPrefixTupleReference(ITypeTraits[] typeTraits, ITypeTraits nullTypeTraits) {
        super(typeTraits, nullTypeTraits);
    }

    // assumes tuple index refers to prefix tuples
    @Override
    public void resetByTupleIndex(ITreeIndexFrame frame, int tupleIndex) {
        BTreeFieldPrefixNSMLeafFrame concreteFrame = (BTreeFieldPrefixNSMLeafFrame) frame;
        IPrefixSlotManager slotManager = concreteFrame.getSlotManager();
        int prefixSlotOff = slotManager.getPrefixSlotOff(tupleIndex);
        int prefixSlot = concreteFrame.getBuffer().getInt(prefixSlotOff);
        setFieldCount(slotManager.decodeFirstSlotField(prefixSlot));
        tupleStartOff = slotManager.decodeSecondSlotField(prefixSlot);
        buf = concreteFrame.getBuffer().array();
        resetByTupleOffset(buf, tupleStartOff);
    }
}
